@@ -32,6 +32,7 @@ struct _DjvuDocument {
 	ddjvu_context_t  *d_context;
 	ddjvu_document_t *d_document;
 	ddjvu_format_t   *d_format;
+	ddjvu_format_t   *thumbs_format;
 
 	gchar            *uri;
 
@@ -44,6 +45,7 @@ struct _DjvuDocument {
 
 int  djvu_document_get_n_pages (EvDocument   *document);
 void djvu_handle_events        (DjvuDocument *djvu_document, 
-			        int           wait);
+			        int           wait,
+				GError      **error);
 
 #endif /* __DJVU_DOCUMENT_INTERNAL_H__ */
